#include <windows.h>
#include <stdio.h>
#include "girder.h"
#include "ammo.h"

#define PlgVersion "1.1"
#define PlgName "ATI Remote Wonder Plugin"
#define PlgDescription "ATI Remote Wonder Plugin for Girder.  Written by karatesnoopy."
#define PlgDeviceID 227

#define WM_REMOTEMSG (WM_APP+1)
DWORD       g_dwThreadID = 0;
HANDLE      g_hThread = NULL;
t_functions g_ActionFunctions;
HINSTANCE   g_hInstance = NULL;
HWND		g_hWnd = NULL;


#define RMCTRL_KEY_OFF					0
#define RMCTRL_KEY_ON					1
#define RMCTRL_KEY_REPEAT				2

///////////////////////////////////////////////////////
// Remote control key defintions
///////////////////////////////////////////////////////

// Numeric Keys

const char* ConvertStateToString( WORD wState )
{
	switch( wState )
	{
		case RMCTRL_KEY_OFF: return "OFF";
		case RMCTRL_KEY_ON:	return "ON";
		case RMCTRL_KEY_REPEAT: return "REPEAT";
	}

	return "unknown";
}

const char* ConvertKeyToString( WORD wKeyEvent )
{

	switch( wKeyEvent )
	{
		case RMCTRL_0: return "0";
		case RMCTRL_1: return "1";
		case RMCTRL_2: return "2";
		case RMCTRL_3: return "3";
		case RMCTRL_4: return "4";
		case RMCTRL_5: return "5";
		case RMCTRL_6: return "6";
		case RMCTRL_7: return "7";
		case RMCTRL_8: return "8";
		case RMCTRL_9: return "9";
		case RMCTRL_HAND: return "HAND";
		case RMCTRL_LEFTMOUSE: return "LEFTMOUSE";
		case RMCTRL_RIGHTMOUSE: return "RIGHTMOUSE";
		case RMCTRL_LEFTMOUSEDOUBLECLICK: return "LEFTMOUSEDOUBLECLICK";
		case RMCTRL_RIGHTMOUSEDOUBLECLICK: return "RIGHTMOUSEDOUBLECLICK";
		case RMCTRL_DIRPADRIGHT: return "DIRPADRIGHT";
		case RMCTRL_DIRPADUP: return "DIRPADUP";
		case RMCTRL_DIRPADLEFT: return "DIRPADLEFT";
		case RMCTRL_DIRPADDOWN: return "DIRPADDOWN";
		case RMCTRL_DIRPADRIGHTUP: return "DIRPADRIGHTUP";
		case RMCTRL_DIRPADRIGHTDOWN: return "DIRPADRIGHTDOWN";
		case RMCTRL_DIRPADLEFTUP: return "DIRPADLEFTUP";
		case RMCTRL_DIRPADLEFTDOWN: return "DIRPADLEFTDOWN";
		case RMCTRL_CHANNELUP: return "CHANNELUP";
		case RMCTRL_CHANNELDOWN: return "CHANNELDOWN";
		case RMCTRL_VOLUMEUP: return "VOLUMEUP";
		case RMCTRL_VOLUMEDOWN: return "VOLUMEDOWN";
		case RMCTRL_MUTE: return "MUTE";
		case RMCTRL_MENU: return "MENU";
		case RMCTRL_SETUP: return "SETUP";
		case RMCTRL_UP: return "UP";
		case RMCTRL_DOWN: return "DOWN";
		case RMCTRL_LEFT: return "LEFT";
		case RMCTRL_RIGHT: return "RIGHT";
		case RMCTRL_ENTER: return "ENTER";
		case RMCTRL_FF: return "FF";
		case RMCTRL_RW: return "RW";
		case RMCTRL_PLAY: return "PLAY";
		case RMCTRL_PAUSE: return "PAUSE";
		case RMCTRL_STOP: return "STOP";
		case RMCTRL_RECORD: return "RECORD";
		case RMCTRL_STOPWATCH: return "STOPWATCH";
		case RMCTRL_A: return "A";
		case RMCTRL_B: return "B";
		case RMCTRL_C: return "C";
		case RMCTRL_D: return "D";
		case RMCTRL_E: return "E";
		case RMCTRL_F: return "F";
		case RMCTRL_RESIZE: return "RESIZE";
		case RMCTRL_WEB_LAUNCH: return "WEB_LAUNCH";
	}

	return "Unknown";
}

// WndProc
LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	if( uMsg == WM_REMOTEMSG )
	{
		if( lParam == RMCTRL_KEY_ON || 
			lParam == RMCTRL_KEY_REPEAT )
		{
			const char* szState;
			const char* szKeyName;
			char szEventString[256];
			szState = ConvertStateToString( (WORD) lParam );
			szKeyName = ConvertKeyToString( (WORD) wParam );

			sprintf( szEventString, "%s:%s", szKeyName, szState );
			g_ActionFunctions.send_event( szEventString, NULL, 0, PlgDeviceID );
		}
	}

    return DefWindowProc( hWnd, uMsg, wParam, lParam );
}


// Girder Plugin Interface
DWORD WINAPI WndThread( LPVOID lpParameter )
{
	MSG msg;

	// Register the windows class
	WNDCLASS wndClass = { 0, WndProc, 0, 0, g_hInstance,
							NULL, 
							LoadCursor( NULL, IDC_ARROW ),
							(HBRUSH)GetStockObject(WHITE_BRUSH),
							NULL, "ATI Remote Wonder Plugin For Grider Class" };
	RegisterClass( &wndClass );

	// Create the render window
	g_hWnd = CreateWindow( "ATI Remote Wonder Plugin For Grider Class", "ATI Remote Wonder Plugin For Grider Window", 
	                        WS_OVERLAPPED,
							CW_USEDEFAULT, CW_USEDEFAULT,
							100, 100, NULL, NULL,
							NULL, 0 );

//	MessageBox(0, "griderplugin: created window", "hi", MB_OK );
	while( GetMessage( &msg, g_hWnd, 0, 0 ) != 0 )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}

//	MessageBox(0, "griderplugin: exiting thread", "hi", MB_OK );

	return 0;
}




// these don't require any involvement by the application
void gir_version(PCHAR data, int size) { strncpy(data,PlgVersion,size); }
void gir_name(PCHAR data, int size) { strncpy(data,PlgName,size); }
void gir_description(PCHAR data, int size) { strncpy(data,PlgDescription,size); }
int gir_devicenum() { return PlgDeviceID; }
int gir_requested_api(int max_api) { return GIR_API_VERSION; }

// called to compare strings
int gir_compare(PCHAR orig, PCHAR recv) { return (strcmp(orig,recv)==0)?GIR_TRUE:GIR_FALSE; }

// called to open the plugin
int gir_open(int gir_major_ver, int gir_minor_ver, int gir_micro_ver, p_functions api_functions )
{
	int ret = GIR_FALSE;
	if (api_functions->size == sizeof(s_functions))
	{
		g_ActionFunctions = *api_functions;
		ret = GIR_TRUE;
	}

	return ret;
}

// called to close the plugin
int gir_close()
{
	return GIR_TRUE;
}

// called to start a device
int gir_start()
{
	g_hThread = CreateThread(NULL,0,&WndThread,g_hInstance,0,&g_dwThreadID);
	if (g_hThread==NULL)
		return GIR_FALSE;

//	MessageBox(0, "girderplug: init success", "hi", MB_OK );
	return GIR_TRUE;
}

// called to stop a device
int gir_stop()
{
//	MessageBox(0, "girderplug: stopping thread", "hi", MB_OK );
	PostThreadMessage( g_dwThreadID, WM_QUIT, 0, 0 );
	PostMessage( g_hWnd, WM_QUIT, 0, 0 );
	WaitForSingleObject( g_hThread, INFINITE );
//	MessageBox(0, "girderplug: exiting", "hi", MB_OK );
	return GIR_TRUE;
}


// Called by windows 
BOOL WINAPI DllMain( HANDLE hModule, 
					DWORD fdwreason,  LPVOID lpReserved )
{

	switch(fdwreason) 
	{
	case DLL_PROCESS_ATTACH:
		g_hInstance=HINSTANCE(hModule);
		break;
	case DLL_THREAD_ATTACH:
		break;
	case DLL_THREAD_DETACH:
		break;
	case DLL_PROCESS_DETACH:
		break;
	}
	return TRUE;
}








